/*
 * @psiControlFrame.java Version 1.1 05/10/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package psi.gui;

import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;

import cmn.cmnStruct;                  // Global Common Data Structure

import gui.guiSearchDirectoryFrame;    // File Search Dialog

import iqstrat.iqstratHeadersStruct;   // Well Header Data Structure

import psi.gui.psiAvailablePanel;      // Available Tracks Panel
import psi.gui.psiControlPanel;        // Pressure Data Panel

import seismic.seisStruct;             // Seismic Data Structure
import seismic.gui.seisPlotFrame;      // Seismic Plot Frame

import util.utilWaitAnimation;         // Wait Animation Dialog
import util.SwingWorker;               // Swing Worker

/** Class psiControlFrame
 *  <p> This Class is the Plot Control Frame for plot frame.
 *
 *  @version 1.1 05/10/2016
 *  @author  John R. Victorine
 */

public class psiControlFrame extends JFrame implements ActionListener, Observer
{
  private static final int _SEISMIC       = seismic.seisTracksStruct._SEISMIC;
  private static final int _SEISMIC_AVG   = seismic.seisTracksStruct._SEISMIC_AVG;
  private static final int _SEISMIC_MAG   = seismic.seisTracksStruct._SEISMIC_MAG;
  private static final int _PRESSURE_TEST = seismic.seisTracksStruct._PRESSURE_TEST;
  private static final int _PRESSURE_AVG  = seismic.seisTracksStruct._PRESSURE_AVG;
  private static final int _PRESSURE_MOD  = seismic.seisTracksStruct._PRESSURE_MOD;
  private static final int _PRESSURE_BARO = seismic.seisTracksStruct._PRESSURE_BARO;

  private static final int SCALE_1    = seismic.seisTracksStruct.SCALE_1;
  private static final int SCALE_2    = seismic.seisTracksStruct.SCALE_2;
  private static final int SCALE_5    = seismic.seisTracksStruct.SCALE_5;
  private static final int SCALE_15   = seismic.seisTracksStruct.SCALE_15;
  private static final int SCALE_30   = seismic.seisTracksStruct.SCALE_30;
  private static final int SCALE_60   = seismic.seisTracksStruct.SCALE_60;
  private static final int SCALE_120  = seismic.seisTracksStruct.SCALE_120;
  private static final int SCALE_180  = seismic.seisTracksStruct.SCALE_180;
  private static final int SCALE_240  = seismic.seisTracksStruct.SCALE_240;
  private static final int SCALE_360  = seismic.seisTracksStruct.SCALE_360;
  private static final int SCALE_720  = seismic.seisTracksStruct.SCALE_720;
  private static final int SCALE_1440 = seismic.seisTracksStruct.SCALE_1440;
  private static final int SCALE_2880 = seismic.seisTracksStruct.SCALE_2880;
  private static final int SCALE_4320 = seismic.seisTracksStruct.SCALE_4320;
  private static final int SCALE_8640 = seismic.seisTracksStruct.SCALE_8640;

  private static final int SCALE[]   = seismic.seisTracksStruct.SCALE;

  // Input Variables

  private cmnStruct          stCMN        = null; // Common Data Structure

  // Global Variables

  private Observable         notifier     = null; // Observables Object

  private iqstratHeadersStruct stWell     = null; // headers Information Data Structure

  private seisStruct         stPSI        = null; // Raw Pressure Data
  private seisStruct         stBaro       = null; // Barometric Pressure Data

  private psiControlPanel    pnlPSI       = null; // Pressure Data Control Panel

  private psiAvailablePanel  pAvailable   = null; // Available Plot Tracks
  private seisPlotFrame      plotframe    = null; // Seismic Data Plot Frame

  // Save Path & Filename

  private String             sPath        = new String("");
  private String             sName        = new String("");

  private guiSearchDirectoryFrame pSearch = null; // Search for Directory to Save PNG

  // -- Time Scale

  private int       iScale  = SCALE_2880; // Minutes / Ince Plot Scale

  private Timestamp tsStart = null; // Starting Date
  private Timestamp tsEnd   = null; // Ending Date

  private Timestamp tsSo    = null; // Original Starting Date
  private Timestamp tsEo    = null; // Original Ending Date

  private static final int START     = 0;
  private static final int END       = 1;
  private static final int MINIMUM   = 2;
  private static final int MAXIMUM   = 3;

  private static final int YEAR      = 0;
  private static final int MONTH     = 1;
  private static final int DAY       = 2;
  private static final int HOUR      = 3;
  private static final int MIN       = 4;
  private static final int SEC       = 5;

  int iData = -1;

  // -- Plot Track Selected

  private int iSelected[]   = { _PRESSURE_TEST };

  // -- Plot Titles

  private String sTitle1 = "";
  private String sTitle2 = "";

  // -- Plot Screen Location & Size

  private int iFrameX       = -1;   // Plot Frame X-Coordintate
  private int iFrameY       = -1;   // Plot Frame Y-Coordintate
  private int iFrameWidth   = -1;   // Plot Frame Width
  private int iFrameHeight  = -1;   // Plot Frame Height

  // -- Screen Width

  private int iControlWidth = 350;  // Control Panel Width
  private int iScreenWidth  = 0;    // Screen Width
  private int iScreenHigh   = 0;    // Scree Hieght

  // Global Frame Widgets

  // -- Date & Time Widgets

  private JTextField txtCursor   = new JTextField(); // Date/Time from mouse

  // .. Start Date & Time

  private JTextField txtYYYYs    = new JTextField(); // Year Field
  private JTextField txtMMs      = new JTextField(); // Month Field
  private JTextField txtDDs      = new JTextField(); // Day Field

  private JTextField txtHHs      = new JTextField(); // 24 Hour Field
  private JTextField txtmms      = new JTextField(); // minute Field
  private JTextField txtssSs     = new JTextField(); // second Field

  // .. End Date & Time

  private JTextField txtYYYYe    = new JTextField(); // Year Field
  private JTextField txtMMe      = new JTextField(); // Month Field
  private JTextField txtDDe      = new JTextField(); // Day Field

  private JTextField txtHHe      = new JTextField(); // 24 Hour Field
  private JTextField txtmme      = new JTextField(); // minute Field
  private JTextField txtssSe     = new JTextField(); // second Field

  // .. Time Control Buttons

  private JButton    btnReset    = new JButton();    // Reset Button
  private JButton    btnModify   = new JButton();    // Modify Button

  // -- Menus
  // .. File Button Menu

  private JMenuItem mPDF    = null; // Create a PDF of Plot
//  private JMenuItem memory   = null; // Display Available Memory Dialog
  private JMenuItem mExit   = null; // Exit Profile Plot

  // .. Depth Scale Menu
/*
  private static final String scale[] =
  {
      "1 min/in",   "2 min/in",  "5 min/in",
     "15 min/in",  "30 min/in", "60 min/in", "120 min/in",
     "180 min/in", "240 min/in"
  };
*/
  private JRadioButtonMenuItem m1    = null; // Display Plot at   1 min / in
  private JRadioButtonMenuItem m2    = null; // Display Plot at   2 min / in
  private JRadioButtonMenuItem m5    = null; // Display Plot at   5 min / in
  private JRadioButtonMenuItem m15   = null; // Display Plot at  15 min / in
  private JRadioButtonMenuItem m30   = null; // Display Plot at  30 min / in
  private JRadioButtonMenuItem m60   = null; // Display Plot at  60 min / in
  private JRadioButtonMenuItem m120  = null; // Display Plot at 120 min / in
  private JRadioButtonMenuItem m180  = null; // Display Plot at 180 min / in
  private JRadioButtonMenuItem m240  = null; // Display Plot at 240 min / in
  private JRadioButtonMenuItem m360  = null; // Display Plot at 360 min / in
  private JRadioButtonMenuItem m720  = null; // Display Plot at 720 min / in
  private JRadioButtonMenuItem m1440 = null; // Display Plot at 1440 min / in
  private JRadioButtonMenuItem m2880 = null; // Display Plot at 2880 min / in
  private JRadioButtonMenuItem m4320 = null; // Display Plot at 4320 min / in
  private JRadioButtonMenuItem m8640 = null; // Display Plot at 8640 min / in

  /** Constructor psiControlFrame( cmnStruct stCMN )
   *  <p> This is the Constructor for this class.
   *  @param stCMN   = Global IQSTRAT Data Structure
   *  @param stWell  = Well Header Data Structure
   *  @param stStart = Starting Date & Time of data set
   *  @param stEnd   = Ending Date & Time of data set
   *  @param stPSI   = Pressure Data Seismic Structure
   */

  public psiControlFrame( cmnStruct stCMN,   iqstratHeadersStruct stWell,
                          Timestamp tsStart, Timestamp tsEnd,
                          seisStruct stPSI )
  {
    try
    {
      this.stCMN   = stCMN;
      this.stWell  = stWell;

//      this.stPSI   = seismic.seisUtility.copyData( stPSI, tsStart, tsEnd, -5 );
      this.stPSI   = stPSI;

      this.tsStart = new Timestamp(tsStart.getTime()); // Starting Date
      this.tsEnd   = new Timestamp(tsEnd.getTime());   // Ending Date

      this.tsSo    = new Timestamp(tsStart.getTime()); // Starting Date
      this.tsEo    = new Timestamp(tsEnd.getTime());   // Ending Date

      jbInit();
      setLimits();
      changeScale( iScale );
//      pnlPSI.compute();

      setConfiguration();

      addWindowListener( new psiControlFrame_WindowListener() );
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlTop            = new JPanel();
    JPanel pnlHeader         = new JPanel();
    JPanel pnlDateTime       = new JPanel();

    JTabbedPane tabbedPane   = new JTabbedPane();

    JMenuBar    menuBar      = new JMenuBar();
    JMenu       menuFile     = new JMenu("File");
    JMenu       menuScale    = new JMenu("Time Scale");
    ButtonGroup groupDepth   = new ButtonGroup();

    // Build Menus

    // -- Menu Bar

    mPDF = new JMenuItem("Create PDF Document Plot");
    mPDF.addActionListener(this);

    mExit = new JMenuItem("Exit");
    mExit.addActionListener(this);

    menuFile.add( mPDF );
    menuFile.addSeparator();
    menuFile.addSeparator();
    menuFile.add( mExit );

    // Time Scale Menu Items

    m1   = new JRadioButtonMenuItem("1 min / in");
    m1.setHorizontalTextPosition(JMenuItem.RIGHT);
    m1.addActionListener(this);
    groupDepth.add(m1);

    m2   = new JRadioButtonMenuItem("2 min / in");
    m2.setHorizontalTextPosition(JMenuItem.RIGHT);
    m2.addActionListener(this);
    groupDepth.add(m2);

    m5   = new JRadioButtonMenuItem("5 min / in");
    m5.setHorizontalTextPosition(JMenuItem.RIGHT);
    m5.addActionListener(this);
    groupDepth.add(m5);

    m15  = new JRadioButtonMenuItem("15 min / in");
    m15.setHorizontalTextPosition(JMenuItem.RIGHT);
    m15.addActionListener(this);
    groupDepth.add(m15);

    m30  = new JRadioButtonMenuItem("30 min / in");
    m30.setHorizontalTextPosition(JMenuItem.RIGHT);
    m30.addActionListener(this);
    groupDepth.add(m30);

    m60  = new JRadioButtonMenuItem("60 min / in");
    m60.setHorizontalTextPosition(JMenuItem.RIGHT);
    m60.addActionListener(this);
    groupDepth.add(m60);

    m120 = new JRadioButtonMenuItem("2 hrs / in");
    m120.setHorizontalTextPosition(JMenuItem.RIGHT);
    m120.addActionListener(this);
    groupDepth.add(m120);

    m180 = new JRadioButtonMenuItem("3 hrs / in");
    m180.setHorizontalTextPosition(JMenuItem.RIGHT);
    m180.addActionListener(this);
    groupDepth.add(m180);

    m240 = new JRadioButtonMenuItem("4 hrs / in");
    m240.setHorizontalTextPosition(JMenuItem.RIGHT);
    m240.addActionListener(this);
    groupDepth.add(m240);

    m360 = new JRadioButtonMenuItem("6 hrs / in");
    m360.setHorizontalTextPosition(JMenuItem.RIGHT);
    m360.addActionListener(this);
    groupDepth.add(m360);

    m720 = new JRadioButtonMenuItem("12 hrs / in");
    m720.setHorizontalTextPosition(JMenuItem.RIGHT);
    m720.addActionListener(this);
    groupDepth.add(m720);

    m1440 = new JRadioButtonMenuItem("1 day / in");
    m1440.setHorizontalTextPosition(JMenuItem.RIGHT);
    m1440.addActionListener(this);
    groupDepth.add(m1440);

    m2880 = new JRadioButtonMenuItem("2 days / in");
    m2880.setSelected(true);
    m2880.setHorizontalTextPosition(JMenuItem.RIGHT);
    m2880.addActionListener(this);
    groupDepth.add(m2880);

    m4320 = new JRadioButtonMenuItem("3 days / in");
    m4320.setHorizontalTextPosition(JMenuItem.RIGHT);
    m4320.addActionListener(this);
    groupDepth.add(m4320);

    m8640 = new JRadioButtonMenuItem("6 days / in");
    m8640.setHorizontalTextPosition(JMenuItem.RIGHT);
    m8640.addActionListener(this);
    groupDepth.add(m8640);

    menuScale.add(m1);
    menuScale.add(m2);
    menuScale.add(m5);
    menuScale.add(m15);
    menuScale.add(m30);
    menuScale.add(m60);
    menuScale.add(m120);
    menuScale.add(m180);
    menuScale.add(m240);
    menuScale.add(m360);
    menuScale.add(m720);
    menuScale.add(m1440);
    menuScale.add(m2880);
    menuScale.add(m4320);
    menuScale.add(m8640);

    menuBar.add(menuFile);
    menuBar.add(menuScale);

    this.setJMenuBar(menuBar);

    // Create an Observable

    notifier = new psiControlFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Other Widgets

    this.setTitle("Plot Control");
    this.getContentPane().setLayout(new BorderLayout());

    // Panel Top

    pnlTop.setLayout(new BorderLayout());

    // -- Header Panel

    pnlDateTime = BuildDateTimePanel();

    // Tabbed Pane

    // -- Pressure Data Panel

    pnlPSI = new psiControlPanel( notifier, stCMN, stPSI, stBaro ); // Pressure Data Control Panel
    tabbedPane.add(pnlPSI,  "Pressure Data");
    tabbedPane.setBackgroundAt(0, new Color(210, 180, 140));

    // -- Available Plot Tracks

    // Create the Available Plot Tracks List and an Observer

    pAvailable = new psiAvailablePanel(notifier);
    getSelected();

    // Add Widgets to Frame

    this.getContentPane().add(pnlTop,       BorderLayout.NORTH);
    pnlTop.add(pnlDateTime,                 BorderLayout.CENTER);

    this.getContentPane().add(tabbedPane,   BorderLayout.CENTER);

    this.getContentPane().add(pAvailable,   BorderLayout.SOUTH);

   // Display the Frame

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    iScreenWidth = d.width;
    iScreenHigh  = d.height-50;

//    this.setSize(new Dimension(iControlWidth, d.height-50));
    this.setSize(new Dimension(iControlWidth, 750));
    this.setLocation(1,1);

//    this.setResizable(false);
    this.setVisible(true);
  }

  /** Method BuildDateTimePanel()
   *  <p> This method will build the date/time panel on the control dialog
   */

  private JPanel BuildDateTimePanel()
  {
    JPanel pnlDateTime       = new JPanel();

    JPanel pnlCursor         = new JPanel();

    JPanel pnlDateTimeCenter = new JPanel();

    JPanel pnlDateTimeStart  = new JPanel();
    JPanel pnlDateTimeEnd    = new JPanel();

    JPanel pnlDateStart      = new JPanel();
    JPanel pnlYYYYs          = new JPanel();
    JPanel pnlMMs            = new JPanel();
    JPanel pnlDDs            = new JPanel();

    JPanel pnlTimeStart      = new JPanel();
    JPanel pnlHHs            = new JPanel();
    JPanel pnlmms            = new JPanel();
    JPanel pnlsss            = new JPanel();

    JPanel pnlDateEnd        = new JPanel();
    JPanel pnlYYYYe          = new JPanel();
    JPanel pnlMMe            = new JPanel();
    JPanel pnlDDe            = new JPanel();

    JPanel pnlTimeEnd        = new JPanel();
    JPanel pnlHHe            = new JPanel();
    JPanel pnlmme            = new JPanel();
    JPanel pnlsse            = new JPanel();

    JPanel pnlTimeButtons    = new JPanel();

    TitledBorder titledBorderDates = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Date & Time(24 Hour) Range");
    titledBorderDates.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderStartDate = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Start Date Time (UTC)");
    titledBorderStartDate.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderEndDate = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "End Date Time (UTC)");
    titledBorderEndDate.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderYYYY = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "YYYY");
    titledBorderYYYY.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMM = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "MM");
    titledBorderYYYY.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderDD = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "DD");
    titledBorderYYYY.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderHH = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "HH");
    titledBorderHH.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBordermm = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "mm");
    titledBordermm.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderssS = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "ss.S");
    titledBorderssS.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // -------------------------------------------------------
    // ------------------- Date Time Panel -------------------
    // -------------------------------------------------------

    pnlDateTime.setLayout(new BorderLayout());
    pnlDateTime.setBorder(titledBorderDates);

    // -- Date Time Cursor Panel

    pnlCursor.setLayout(new BorderLayout());
    pnlCursor.add(txtCursor, BorderLayout.CENTER);

    txtCursor.setText("");
    txtCursor.setEditable(false);

    // -- Date Time Center Panel

    pnlDateTimeCenter.setLayout(new GridLayout(2,1));

    // .......................................................
    // ................... Date Time Start ...................
    // .......................................................

    pnlDateTimeStart.setLayout(new GridLayout());
    pnlDateTimeStart.setBorder(titledBorderStartDate);

    // -- Date Start Panel

    pnlDateStart.setLayout(new BorderLayout());

    // .. Year Textfield Panel

    pnlYYYYs.setLayout(new BorderLayout());
    pnlYYYYs.setBorder(titledBorderYYYY);
    pnlYYYYs.add(txtYYYYs,  BorderLayout.CENTER);

    txtYYYYs.setText("");
    txtYYYYs.setColumns(4);
	txtYYYYs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtYYYYs.addFocusListener(new psiControlFrameFocusAdapter(this));

    // .. Month Textfield Panel

    pnlMMs.setLayout(new BorderLayout());
    pnlMMs.setBorder(titledBorderMM);
    pnlMMs.add(txtMMs,  BorderLayout.CENTER);

    txtMMs.setText("");
    txtMMs.setColumns(2);
	txtMMs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMMs.addFocusListener(new psiControlFrameFocusAdapter(this));

    // .. Day Textfield Panel

    pnlDDs.setLayout(new BorderLayout());
    pnlDDs.setBorder(titledBorderDD);
    pnlDDs.add(txtDDs,  BorderLayout.CENTER);

    txtDDs.setText("");
    txtDDs.setColumns(2);
	txtDDs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtDDs.addFocusListener(new psiControlFrameFocusAdapter(this));

    // -- Attach YYYY,MM,DD Panels to Date Start Panel

    pnlDateStart.add(pnlYYYYs, BorderLayout.WEST);
    pnlDateStart.add(pnlMMs,   BorderLayout.CENTER);
    pnlDateStart.add(pnlDDs,   BorderLayout.EAST);

    // -- Time Start Panel

    pnlTimeStart.setLayout(new BorderLayout());

    // .. Hour Textfield Panel

    pnlHHs.setLayout(new BorderLayout());
    pnlHHs.setBorder(titledBorderHH);
    pnlHHs.add(txtHHs, BorderLayout.CENTER);

    txtHHs.setText("");
    txtHHs.setColumns(2);
	txtHHs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtHHs.addFocusListener(new psiControlFrameFocusAdapter(this));

    // .. Minutes Textfield Panel

    pnlmms.setLayout(new BorderLayout());
    pnlmms.setBorder(titledBordermm);
    pnlmms.add(txtmms, BorderLayout.CENTER);

    txtmms.setText("");
    txtmms.setColumns(2);
	txtmms.setHorizontalAlignment(SwingConstants.TRAILING);
	txtmms.addFocusListener(new psiControlFrameFocusAdapter(this));

    // .. Seconds Textfield Panel

    pnlsss.setLayout(new BorderLayout());
    pnlsss.setBorder(titledBorderssS);
    pnlsss.add(txtssSs, BorderLayout.CENTER);

    txtssSs.setText("");
    txtssSs.setColumns(4);
	txtssSs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtssSs.addFocusListener(new psiControlFrameFocusAdapter(this));

    // -- Attach HH,mm,ss.SS Panels to Date Start Panel

    pnlTimeStart.add(pnlHHs, BorderLayout.WEST);
    pnlTimeStart.add(pnlmms, BorderLayout.CENTER);
    pnlTimeStart.add(pnlsss, BorderLayout.EAST);

    // .......................................................
    // .................... Date Time End ....................
    // .......................................................

    pnlDateTimeEnd.setLayout(new GridLayout());
    pnlDateTimeEnd.setBorder(titledBorderEndDate);

    // -- Date End Panel

    pnlDateEnd.setLayout(new BorderLayout());

    // .. Year Textfield Panel

    pnlYYYYe.setLayout(new BorderLayout());
    pnlYYYYe.setBorder(titledBorderYYYY);
    pnlYYYYe.add(txtYYYYe,  BorderLayout.CENTER);

    txtYYYYe.setText("");
    txtYYYYe.setColumns(4);
	txtYYYYe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtYYYYe.addFocusListener(new psiControlFrameFocusAdapter(this));

    // .. Month Textfield Panel

    pnlMMe.setLayout(new BorderLayout());
    pnlMMe.setBorder(titledBorderMM);
    pnlMMe.add(txtMMe,  BorderLayout.CENTER);

    txtMMe.setText("");
    txtMMe.setColumns(2);
	txtMMe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMMe.addFocusListener(new psiControlFrameFocusAdapter(this));

    // .. Day Textfield Panel

    pnlDDe.setLayout(new BorderLayout());
    pnlDDe.setBorder(titledBorderDD);
    pnlDDe.add(txtDDe,  BorderLayout.CENTER);

    txtDDe.setText("");
    txtDDe.setColumns(2);
	txtDDe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtDDe.addFocusListener(new psiControlFrameFocusAdapter(this));

    // -- Attach YYYY,MM,DD Panels to Date Start Panel

    pnlDateEnd.add(pnlYYYYe, BorderLayout.WEST);
    pnlDateEnd.add(pnlMMe,   BorderLayout.CENTER);
    pnlDateEnd.add(pnlDDe,   BorderLayout.EAST);

    // -- Time End Panel

    pnlTimeEnd.setLayout(new BorderLayout());

    // .. Hour Textfield Panel

    pnlHHe.setLayout(new BorderLayout());
    pnlHHe.setBorder(titledBorderHH);
    pnlHHe.add(txtHHe, BorderLayout.CENTER);

    txtHHe.setText("");
    txtHHe.setColumns(2);
	txtHHe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtHHe.addFocusListener(new psiControlFrameFocusAdapter(this));

    // .. Minutes Textfield Panel

    pnlmme.setLayout(new BorderLayout());
    pnlmme.setBorder(titledBordermm);
    pnlmme.add(txtmme, BorderLayout.CENTER);

    txtmme.setText("");
    txtmme.setColumns(2);
	txtmme.setHorizontalAlignment(SwingConstants.TRAILING);
	txtmme.addFocusListener(new psiControlFrameFocusAdapter(this));

    // .. Seconds Textfield Panel

    pnlsse.setLayout(new BorderLayout());
    pnlsse.setBorder(titledBorderssS);
    pnlsse.add(txtssSe, BorderLayout.CENTER);

    txtssSe.setText("");
    txtssSe.setColumns(4);
	txtssSe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtssSe.addFocusListener(new psiControlFrameFocusAdapter(this));

    // -- Attach HH,mm,ss.SS Panels to Date Start Panel

    pnlTimeEnd.add(pnlHHe, BorderLayout.WEST);
    pnlTimeEnd.add(pnlmme, BorderLayout.CENTER);
    pnlTimeEnd.add(pnlsse, BorderLayout.EAST);

    // .......................................................
    // .................. Date Time Buttons ..................
    // .......................................................

    pnlTimeButtons.setLayout(new GridLayout());

    btnReset.setFont(new java.awt.Font("Dialog", 1, 11));
    btnReset.setPreferredSize(new Dimension(67, 20));
    btnReset.setText("Reset");
    btnReset.addActionListener(this);

    btnModify.setFont(new java.awt.Font("Dialog", 1, 11));
    btnModify.setPreferredSize(new Dimension(71, 20));
    btnModify.setText("Modify");
    btnModify.addActionListener(this);

    // Attach Widgets to Panel

//    pnlDateTime.add(pnlCursor,              BorderLayout.NORTH);

    pnlDateTime.add(pnlDateTimeCenter,      BorderLayout.CENTER);
    pnlDateTimeCenter.add(pnlDateTimeStart, null);
    pnlDateTimeStart.add(pnlDateStart,      null);
    pnlDateTimeStart.add(pnlTimeStart,      null);

    pnlDateTimeCenter.add(pnlDateTimeEnd,   null);
    pnlDateTimeEnd.add(pnlDateEnd,          null);
    pnlDateTimeEnd.add(pnlTimeEnd,          null);

    pnlDateTime.add(pnlTimeButtons,         BorderLayout.SOUTH);
    pnlTimeButtons.add(btnReset,            null);
    pnlTimeButtons.add(btnModify,           null);

    return (pnlDateTime);
  }

  /* ===================================================================== *
   * ----------------------------- CLOSE --------------------------------- *
   * ===================================================================== */

  /** METHOD psiControlFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class psiControlFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e) { close(); }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    stCMN     = null; // Global IQSTRAT Data Structure

    // Global Variables

    notifier  = null; // Observables Object

    stWell    = null; // headers Information Data Structure

    if (stPSI != null)
      stPSI.delete();
    stPSI    = null; // Pressure Seismic Data Structure

    if (stBaro != null)
      stBaro.delete();
    stBaro = null;

    if (plotframe != null)
      plotframe.close();
    plotframe = null; // Seismic Data Plot Frame

    if (pSearch != null)
      pSearch.close();
    pSearch   = null;

    if (pnlPSI != null)
      pnlPSI.close();
    pnlPSI = null;

    // Save Path & Filename

    sPath      = null;
    sName      = null;

    // Time Scale

    tsStart   = null; // Starting Date
    tsEnd     = null; // Ending Date

    tsSo      = null; // Original Starting Date
    tsEo      = null; // Original Ending Date

    // -- Plot Titles

    sTitle1 = null;
    sTitle2 = null;

    // Plot Track Selected

    iSelected = null; // Selected Tracks

    // Global Frame Widgets

    // -- Date & Time Widgets

    txtCursor   = null; // Date/Time from mouse

    // .. Start Date & Time

    txtYYYYs    = null; // Year Field
    txtMMs      = null; // Month Field
    txtDDs      = null; // Day Field

    txtHHs      = null; // 24 Hour Field
    txtmms      = null; // minute Field
    txtssSs     = null; // second Field

    // .. End Date & Time

    txtYYYYe    = null; // Year Field
    txtMMe      = null; // Month Field
    txtDDe      = null; // Day Field

    txtHHe      = null; // 24 Hour Field
    txtmme      = null; // minute Field
    txtssSe     = null; // second Field

    // .. Time Control Buttons

    btnReset    = null;    // Reset Button
    btnModify   = null;    // Modify Button

    // -- Menus

    mPDF      = null; // PDF of Profile Plot
    mExit     = null; // Exit

    // -- Depth Scale Buttons

    m1        = null; // Display Plot at    1 min / in
    m2        = null; // Display Plot at    2 min / in
    m5        = null; // Display Plot at    5 min / in
    m15       = null; // Display Plot at   15 min / in
    m30       = null; // Display Plot at   30 min / in
    m60       = null; // Display Plot at   60 min / in
    m120      = null; // Display Plot at  120 min / in
    m180      = null; // Display Plot at  180 min / in
    m240      = null; // Display Plot at  240 min / in
    m360      = null; // Display Plot at  360 min / in
    m720      = null; // Display Plot at  720 min / in
    m1440     = null; // Display Plot at 1440 min / in
    m2880     = null; // Display Plot at 2880 min / in
    m4320     = null; // Display Plot at 4320 min / in
    m8640     = null; // Display Plot at 8640 min / in

    dispose();
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getSelected()
   * <p> This method will retrieve the selected tracks
   */

  private void getSelected() { iSelected = pAvailable.getSelected(); }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setEvent()
   *  <p> This method will set the event information at the top of the profile plot
   */

  public void setEvent( String sCentral,
                        String sUTC,
                        double dMag,
                        double dLatitude,
                        double dLongitude,
                        double depth,
                        String source )
  {
	String sTitle  = "";
	String sTitle2 = "";

	sTitle  = new String("Magnitude "+ dMag + " Seismic Event at Central Time: " + sCentral +
	                     " Coordinated Universal Time (UTC): " + sUTC );
    sTitle2 = new String("Latitude: " + dLatitude + " Longitude: " + dLongitude +
                         " Depth: " + depth + " (km)" + " Source: " + source);

    plotframe.setTitles( sTitle, sTitle2 );
  }

  /** Method setLimits()
   * <p> This method will set the Seismic Data Structure
   * @param st - Seismic Data Structure
   */

  public void setLimits()
  {
	long lstart = (long) 0;
	long lend   = (long) 0;
	long diff   = (long) 0;

    setDateTime(START,  tsStart);
    setDateTime(END,    tsEnd);

    lstart = tsStart.getTime();
    lend   = tsEnd.getTime();
    diff   = lend - lstart;

    if (diff <= SCALE[SCALE_120])
    {
      iScale = SCALE_15;
      m15.setSelected(true);
    }

    if (diff <= SCALE[SCALE_60])
    {
      iScale = SCALE_5;
      m5.setSelected(true);
    }

    if (diff <= SCALE[SCALE_30])
    {
      iScale = SCALE_2;
      m2.setSelected(true);
    }

    if (diff <= SCALE[SCALE_15])
    {
      iScale = SCALE_1;
      m1.setSelected(true);
    }

    plot();
  }

  /** Method changeScale()
   * <p> This method will change the time Scale.
   * @param iScale = The Time Scale
   */

  private void changeScale(int iScale)
  {
    this.iScale = iScale;

//    lblScale.setText("Depth Scale: "+ scale[iScale]);

    setScale(iScale, tsEnd, tsStart);
  }

  /** Method setScale(int iScale, double dMaximum, double dMinimum)
   * <p> This method will set the Plot scale in the
   *     Cross Plot Data List Structure
   * @param iScale    = The Time Track Scale identifier
   * @param tsMaximum = The Maximum Timestamp
   * @param tsMinimum = The Minimum Timestamp
   */

  private void setScale(int iScale, Timestamp tsMaximum, Timestamp tsMinimum)
  {
    if (plotframe != null)
    {
  	  sTitle2 = new String("Pressure measured from " + tsStart.toString() + " to " + tsEnd.toString());
      plotframe.setTitles(sTitle1, sTitle2);

      plotframe.setScale( iScale, tsMaximum, tsMinimum );
    }
  }

  /** Method setDateTime()
   *  <p> This method will set the Date and Time Value into either the
   *      Start Date Time or End Date Time.
   *  @param iDate = 0 for Start, 1 for End
   *  @param ts    = timestamp
   */

  private void setDateTime(int iDate, Timestamp ts)
  {
    int i = 0;
    String tokens[]    = null;
    String sDelimiter  = new String("[ ]+");
    String sDelimiter1 = new String("[-]+");
    String sDelimiter2 = new String("[:]+");
    String str         = "";
    String sDate       = "";
    String sTime       = "";
    String date[]      = { "", "", "" };
    String time[]      = { "", "", "" };
    int    iContinue   = 0;
    double d           = 0.0;
    int    id          = 0;

    if (ts != null)
    {
      str = new String(ts.toString());
      tokens = str.split( sDelimiter );
      if (tokens.length > 0)
        sDate = new String(tokens[0].trim());

      tokens = sDate.split( sDelimiter1 );
      if (tokens.length == 3)
      {
  	    for (i=0; i<3; i++)
	    {
	      date[i] = new String(tokens[i]);
	    }

	    iContinue = 1;

        if (iContinue == 1)
	    {
	      iContinue = 0;

	      str = new String(ts.toString());
	      tokens = str.split( sDelimiter );
	      if (tokens.length > 0)
	        sTime = new String(tokens[1].trim());

          tokens = sTime.split( sDelimiter2 );
          if (tokens.length == 3)
          {
  	        for (i=0; i<3; i++)
  	        {
	          time[i] = new String(tokens[i]);
	        }

	        iContinue = 1;

	        if (iContinue == 1)
	        {
	          switch (iDate)
	          {
	            case START:
	              txtYYYYs.setText(date[0]);
	              txtMMs.setText(date[1]);
	              txtDDs.setText(date[2]);

	              txtHHs.setText(time[0]);
	              txtmms.setText(time[1]);
	              txtssSs.setText(time[2]);
	              break;

	            case END:
	              txtYYYYe.setText(date[0]);
	              txtMMe.setText(date[1]);
	              txtDDe.setText(date[2]);

	              txtHHe.setText(time[0]);
	               txtmme.setText(time[1]);
	              txtssSe.setText(time[2]);
	              break;
	          }
	        }
  	      }
	    }
      }
    }
  }

  /** Method setConfiguration()
   * <p> This method will set the Available Track Configuration
   */

  private void setConfiguration()
  {
    seisStruct stData = null;

    pAvailable.setByAction( 1,
                            stPSI,
                            stData, //pnlPSI.getAvgData(),
                            stBaro,
                            stData,
                            stData );
    getSelected();
    plotframe.setSelected(iSelected);
  }

  /* ===================================================================== *
   * -------------------------- PLOT METHODS ----------------------------- *
   * ===================================================================== */

  /** Method plot()
   *  <p> This method will plot the seismic data
   */

  private void plot()
  {
    String    sTitle  = "Plot Seismic File Data";
    String    sTitle1 = "";
    String    sTitle2 = "";

    if (plotframe != null)
    {
      iFrameX      = plotframe.getLocation().x;
      iFrameY      = plotframe.getLocation().y;
      iFrameWidth  = plotframe.getSize().width;
      iFrameHeight = plotframe.getSize().height;

      plotframe.close();
      plotframe = null;
    }

    plotframe = new seisPlotFrame( stCMN,
                                   tsStart,
	                               tsEnd,
	                               iScale,
                                   iSelected,
                                   iControlWidth,
                                   iFrameX,
                                   iFrameY,
                                   iFrameWidth,
                                   iFrameHeight );

    if (stWell != null)
    {
      sTitle1 = new String(stWell.sName + " (" + stWell.sAPI  + ")  ");

      if (stWell.dLatitude != 0.0)
        sTitle1 = new String( sTitle1 + "Lat: " +
                              stWell.dLatitude + " " );

      if (stWell.dLongitude != 0.0)
        sTitle1 = new String( sTitle1 + "Long: " +
                              stWell.dLongitude + " " );

      if (stWell.dGL > 0.0)
        sTitle1 = new String( sTitle1 + "GL: " +
                              stWell.dGL + " " );
      else if (stWell.dKB > 0.0)
        sTitle1 = new String( sTitle1 + "KB: " +
                              stWell.dKB + " " );
	}

	sTitle2 = new String("Pressure measured from " + tsStart.toString() + " to " + tsEnd.toString());
    plotframe.setTitles(sTitle1, sTitle2);

    plotframe.setData(_PRESSURE_TEST, stPSI);  // Pressure Seismic Data structure
//    plotframe.setData(_PRESSURE_BARO, stBaro);  // Pressure Seismic Data structure
//    plotframe.setBaro(_PRESSURE_MOD,  stBaro);  // Pressure Seismic Data structure

    if (pnlPSI  != null) { pnlPSI.setPlot( plotframe ); }
  }

  /** Method setFront()
   * <p> This method will set both the Control and Plot Frames to the front
   */

  public void setFront()
  {
    this.toFront();
    if (plotframe != null) plotframe.toFront();
  }

  /* ===================================================================== *
   * -------------------------- PDF ACTIONS ------------------------------ *
   * ===================================================================== */

  /** Method createPDF()
   *  <p> This method will create a PDF Image of Production Plot
   */

  private void createPDF()
  {
    seismic.io.seisIO_Files.createImage( sPath, sName, plotframe.getImage() );
    seismic.io.seisIO_Files.createHTML( sPath, sName );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    // Set Time Scale

    if (event.getSource() == m8640) { changeScale(SCALE_8640); }
    if (event.getSource() == m4320) { changeScale(SCALE_4320); }
    if (event.getSource() == m2880) { changeScale(SCALE_2880); }
    if (event.getSource() == m1440) { changeScale(SCALE_1440); }
    if (event.getSource() == m720)  { changeScale(SCALE_720); }
    if (event.getSource() == m360)  { changeScale(SCALE_360); }
    if (event.getSource() == m240)  { changeScale(SCALE_240); }
    if (event.getSource() == m180)  { changeScale(SCALE_180); }
    if (event.getSource() == m120)  { changeScale(SCALE_120); }
    if (event.getSource() == m60)   { changeScale(SCALE_60);  }
    if (event.getSource() == m30)   { changeScale(SCALE_30);  }
    if (event.getSource() == m15)   { changeScale(SCALE_15);  }
    if (event.getSource() == m5)    { changeScale(SCALE_5);   }
    if (event.getSource() == m2)    { changeScale(SCALE_2);   }
    if (event.getSource() == m1)    { changeScale(SCALE_1);   }

    if (event.getSource() == mPDF) // Create a PDF of Profile Plot
    {
	  if (sPath.length() == 0)
        sPath = seismic.io.seisIO_Files.getHomeDir();

      if (sName.length() == 0)
        sName = new String( "seismic" );

      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".png");
    }

//    if (event.getSource() == btnFreq) { displayFrequency();	}

    if (event.getSource() == btnReset)    // Reset Button
    {
      long  lstart = tsSo.getTime();
      long  lend   = tsEo.getTime();
      long  diff   = lend - lstart;

      if (diff <= SCALE[SCALE_120])
      {
        iScale = SCALE_15;
        m15.setSelected(true);
	  }

      if (diff <= SCALE[SCALE_60])
      {
        iScale = SCALE_5;
        m5.setSelected(true);
	  }

      if (diff <= SCALE[SCALE_30])
      {
        iScale = SCALE_2;
        m2.setSelected(true);
	  }

      if (diff <= SCALE[SCALE_15])
      {
        iScale = SCALE_1;
        m1.setSelected(true);
	  }

      setDateTime(START, tsSo);
      setDateTime(END,   tsEo);

      setScale( iScale, tsEo, tsSo );
	}

    if (event.getSource() == btnModify)    // Modify Button
    {
      SimpleDateFormat dfm   = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSSSSS");
      String           sText = "";
      long             ltime = (long) 0;

      sText   = new String( txtYYYYs.getText() + "-" +
		                    txtMMs.getText()   + "-" +
		                    txtDDs.getText()   + " " +
		                    txtHHs.getText()   + ":" +
		                    txtmms.getText()   + ":" +
		                    txtssSs.getText() );

      try
      {
        ltime   = dfm.parse(sText).getTime();
        tsStart = new Timestamp( ltime );
	  }
      catch (ParseException e)
      {
        e.printStackTrace();
      }

      sText   = new String( txtYYYYe.getText() + "-" +
		                    txtMMe.getText()   + "-" +
		                    txtDDe.getText()   + " " +
		                    txtHHe.getText()   + ":" +
		                    txtmme.getText()   + ":" +
		                    txtssSe.getText() );

      try
      {
        ltime   = dfm.parse(sText).getTime();
        tsEnd   = new Timestamp( ltime );
	  }
      catch (ParseException e)
      {
        e.printStackTrace();
      }

      setScale( iScale, tsEnd, tsStart );
	}

    // Exit

    if (event.getSource() == mExit)  { close(); }
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
//	focusLostTime(e);
  }

  /** METHOD focusLostTime()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostTime(FocusEvent e)
  {
    int     iNumeric = 0;
    int     len      = 0;
    int     iField   = -1;
    boolean bVal     = true;
    String  sMessage = new String("");
    String  sTemp    = new String("");
    int     id       = 0;
    double  d        = 0.0;
    double  d1       = 0.0;

    if (e.getSource() == txtYYYYs)
    {
      iData    = START;
      iField   = YEAR;
      iNumeric = 1;
      sTemp    = new String(txtYYYYs.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Year Value is a Numeric Field");
    }

    if (e.getSource() == txtMMs)
    {
      iData    = START;
      iField   = MONTH;
      iNumeric = 1;
      sTemp    = new String(txtMMs.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Month Value is a Numeric Field");
    }

    if (e.getSource() == txtDDs)
    {
	  iData    = START;
	  iField   = DAY;
      iNumeric = 1;
      sTemp    = new String(txtDDs.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Day Value is a Numeric Field");
    }

    if (e.getSource() == txtHHs)
    {
      iData    = START;
      iField   = HOUR;
      iNumeric = 1;
      sTemp    = new String(txtHHs.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Hour Value is a Numeric Field");
    }

    if (e.getSource() == txtmms)
    {
      iData    = START;
      iField   = MIN;
      iNumeric = 1;
      sTemp    = new String(txtmms.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Minutes Value is a Numeric Field");
    }

    if (e.getSource() == txtssSs)
    {
      iData    = START;
      iField   = SEC;
      iNumeric = 1;
      sTemp    = new String(txtssSs.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Seconds Value is a Numeric Field");
    }

    if (e.getSource() == txtYYYYe)
    {
      iData    = END;
      iField   = YEAR;
      iNumeric = 1;
      sTemp    = new String(txtYYYYe.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Emd Year Value is a Numeric Field");
    }

    if (e.getSource() == txtMMe)
    {
      iData    = END;
      iField   = MONTH;
      iNumeric = 1;
      sTemp    = new String(txtMMe.getText().trim());
      len      = sTemp.length();
      sMessage = new String("End Month Value is a Numeric Field");
    }

    if (e.getSource() == txtDDe)
    {
      iData    = END;
      iField   = DAY;
      iNumeric = 1;
      sTemp    = new String(txtDDe.getText().trim());
      len      = sTemp.length();
      sMessage = new String("End Day Value is a Numeric Field");
    }

    if (e.getSource() == txtHHe)
    {
      iData    = END;
      iField   = HOUR;
      iNumeric = 1;
      sTemp    = new String(txtHHe.getText().trim());
      len      = sTemp.length();
      sMessage = new String("End Hour Value is a Numeric Field");
    }

    if (e.getSource() == txtmme)
    {
      iData    = END;
      iField   = MIN;
      iNumeric = 1;
      sTemp    = new String(txtmme.getText().trim());
      len      = sTemp.length();
      sMessage = new String("End Minutes Value is a Numeric Field");
    }

    if (e.getSource() == txtssSe)
    {
      iData    = END;
      iField   = SEC;
      iNumeric = 1;
      sTemp    = new String( txtssSe.getText().trim());
      len      = sTemp.length();
      sMessage = new String("End Seconds Value is a Numeric Field");
    }

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);

        switch (iData)
        {
          case START:
            setDateTime(START, tsStart);
            break;

          case END:
            setDateTime(END,   tsEnd);
            break;
	}
      }
      else
      {
        if (cmn.cmnString.isNumeric(sTemp))
        {
          switch (iField)
          {
            case YEAR:
            case MONTH:
            case DAY:
            case HOUR:
            case MIN:
    	      id = cmn.cmnString.stringToInt(sTemp);
              break;
            case SEC:
              d  = cmn.cmnString.stringToDouble(sTemp);
              break;
 	  }
        }

        switch (iData)
        {
          case START:
            switch (iField)
            {
              case YEAR:
                if (len != 4) { bVal = false; }
                break;
              case MONTH:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 12)) { bVal = false; }
                else if (len < 2) { txtMMs.setText("0"+sTemp); }
                break;
              case DAY:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 31)) { bVal = false; }
                else if (len < 2) { txtDDs.setText("0"+sTemp); }
                break;
              case HOUR:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 24)) { bVal = false; }
                else if (len < 2) { txtHHs.setText("0"+sTemp); }
                break;
              case MIN:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 60)) { bVal = false; }
                else if (len < 2) { txtmms.setText("0"+sTemp); }
                break;
              case SEC:
                if ((d < 0.0) || (d >= 60.0 )) { bVal = false; }
                break;
	    }

	    if (!bVal)
              setDateTime(START, tsStart);
            break;

          case END:
  	    switch (iField)
            {
              case YEAR:
                if (len != 4) { bVal = false; }
                  bVal = false;
                break;
              case MONTH:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 12)) { bVal = false; }
                else if (len < 2) { txtMMe.setText("0"+sTemp); }
                break;
              case DAY:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 31)) { bVal = false; }
                else if (len < 2) { txtDDe.setText("0"+sTemp); }
                break;
              case HOUR:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 24)) { bVal = false; }
                else if (len < 2) { txtHHe.setText("0"+sTemp); }
                break;
              case MIN:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 60)) { bVal = false; }
                else if (len < 2) { txtmme.setText("0"+sTemp); }
                break;
              case SEC:
                if ((d < 0.0) || (d >= 60.0 )) { bVal = false; }
                break;
	    }

	    if (!bVal)
              setDateTime(END,   tsEnd);
            break;
 	}
      }
    }
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String( (String) arg);

    if (sArg.equals("Average Pressure Complete"))
    {
//      if (pnlPSI != null) {	pnlPSI.shareData();  }
	}

    if (sArg.equals("Available Track Panel Changed"))
    {
      getSelected();
      if (plotframe != null)
      {
        plotframe.setSelected(iSelected);
      }
    }

    if (sArg.equals("WAVELET Completed"))
    {
	}

    if (sArg.equals("Removed Tidal Pressure Wave"))
    {
//	  if (pnlPSI != null) {	pnlPSI.getModifiedData(); }
	}

    if (sArg.equals("Directory Path Selected"))
    {
      if (pSearch != null)
      {
        sPath = new String( pSearch.getPath() );
        sName = new String( pSearch.getFile() );
      }

      createPDF();

      if (pSearch != null)
        pSearch.close();
      pSearch = null;
	}
  }
}

/** CLASS psiControlFrameFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class psiControlFrameFocusAdapter extends java.awt.event.FocusAdapter
{
  psiControlFrame adaptee;

  psiControlFrameFocusAdapter(psiControlFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/** CLASS psiControlFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class psiControlFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 05/10/2016
 *  @author  John Victorine
 */

