/*
 * @psiControlPanel.java Version 1.1 05/11/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package psi.gui;

import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import cmn.cmnStruct;             // Global Common Data Structure

import seismic.seisStruct;        // Seismic Data Structure
import seismic.gui.seisPlotFrame; // Seismic Plot Frame

import psi.psiTidalStruct;        // Tidal Parameters Data Structure
//import psi.gui.psiTidesFrame;     // Compute Tidal Pressure Frame
//import psi.gui.psiSmoothingPanel; // Smooth Seismic Wave Signal

/** Class psiControlPanel
 *  <p> This Class is the Plot Control Frame for plot frame.
 *
 *  @version 1.1 05/11/2016
 *  @author  John R. Victorine
 */

public class psiControlPanel extends JPanel implements ActionListener
{
  private static final int _PRESSURE_TEST = seismic.seisTracksStruct._PRESSURE_TEST;
  private static final int _PRESSURE_AVG  = seismic.seisTracksStruct._PRESSURE_AVG;
  private static final int _PRESSURE_MOD  = seismic.seisTracksStruct._PRESSURE_MOD;
  private static final int _PRESSURE_BARO = seismic.seisTracksStruct._PRESSURE_BARO;

  // Input Variables

  private cmnStruct          stCMN        = null; // Common Data Structure
  private Observable         notifier     = null; // Observables Object
  private seisStruct         stData       = null; // Raw Seismic Data
  private seisStruct         stMOD        = null; // Normalized to Mean value
  private seisStruct         stBaro       = null; // Barometric Pressure Seismic Data

  // Global Variables

  private Timestamp tsStart = null; // Starting Date
  private Timestamp tsEnd   = null; // Ending Date

//  private psiSmoothingPanel pnlWave       = null; // Constructed Wave
//  private psiTidesFrame     pnlFrequency  = null; // Compute the Tidal Frequecy
  private seisPlotFrame     plotframe     = null; // Seismic Data Plot Frame

  private static final int MINIMUM   = 2;
  private static final int MAXIMUM   = 3;

  int iData = -1;

  // Global Frame Widgets

  // -- Track 1 Minimum & Maximum Limits Widgets

  private JTextField txtMinimum1 = new JTextField(); // Minimum Value
  private JTextField txtMaximum1 = new JTextField(); // Maximum Value

  // -- Track 1 Minimum & Maximum Limits Widgets

  private JTextField txtMinimumB = new JTextField(); // Minimum Value
  private JTextField txtMaximumB = new JTextField(); // Maximum Value

  // -- Frequency Plot Button

  private JButton    btnFilter   = new JButton();    // Filter Data Button
  private JButton    btnFreq     = new JButton();    // Frequency Button
  private JButton    btnWave     = new JButton();    // Wavelet Button

  /** Constructor psiControlPanel()
   *  <p> This is the Constructor for this class.
   *  @param notifier = Events Observable Class
   *  @param stCMN    = Global IQSTRAT Data Structure
   *  @param stData   = Raw Pressure Data
   */

  public psiControlPanel( Observable notifier, cmnStruct stCMN, seisStruct stData )
  {
	int iStart = 0;

    try
    {
      this.notifier = notifier;
      this.stCMN    = stCMN;
      this.stData   = stData;

      jbInit();

      if (stData != null)
      {
        this.tsStart = new Timestamp(stData.tsStart.getTime()); // Starting Date
        this.tsEnd   = new Timestamp(stData.tsEnd.getTime());   // Ending Date

        for (int i=0; i<stData.iRows; i++)
        {
  	      if ((iStart == 0) && (stData.data[i] > 0.0))
  	      {
	        stData.dMinimum = stData.data[i];
	        stData.dMaximum = stData.data[i];
	        iStart = 1;
	      }

          if (iStart > 0)
          {
	        if ((stData.dMinimum > stData.data[i]) && (stData.data[i] > 0.0))
	          stData.dMinimum = stData.data[i];

	        if (stData.dMaximum < stData.data[i])
	          stData.dMaximum = stData.data[i];
		  }
	    }

        txtMinimum1.setText("" + stData.dMinimum);
        txtMaximum1.setText("" + stData.dMaximum);
      }
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** Constructor psiControlPanel()
   *  <p> This is the Constructor for this class.
   *  @param notifier = Events Observable Class
   *  @param stCMN    = Global IQSTRAT Data Structure
   *  @param stData   = Raw Pressure Data
   *  @param stBaro   = Barometric Pressure Data
   */

  public psiControlPanel( Observable notifier, cmnStruct stCMN,
                          seisStruct stData, seisStruct stBaro )
  {
	int iStart = 0;

    try
    {
      this.notifier = notifier;
      this.stCMN    = stCMN;
      this.stData   = stData;
      this.stBaro   = stBaro;

      jbInit();

      if (stData != null)
      {
        this.tsStart = new Timestamp(stData.tsStart.getTime()); // Starting Date
        this.tsEnd   = new Timestamp(stData.tsEnd.getTime());   // Ending Date

        for (int i=0; i<stData.iRows; i++)
        {
  	      if ((iStart == 0) && (stData.data[i] > 0.0))
  	      {
	        stData.dMinimum = stData.data[i];
	        stData.dMaximum = stData.data[i];
	        iStart = 1;
	      }

          if (iStart > 0)
          {
	        if ((stData.dMinimum > stData.data[i]) && (stData.data[i] > 0.0))
	          stData.dMinimum = stData.data[i];

	        if (stData.dMaximum < stData.data[i])
	          stData.dMaximum = stData.data[i];
		  }
	    }

        txtMinimum1.setText("" + stData.dMinimum);
        txtMaximum1.setText("" + stData.dMaximum);
      }

      if (stBaro != null)
      {
        txtMinimumB.setText("" + stBaro.dMinimum);
        txtMaximumB.setText("" + stBaro.dMaximum);
	  }
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlTop            = new JPanel();
    JPanel pnlHeader         = new JPanel();

    JPanel pnlCenter         = new JPanel();
    JPanel pnlTrack1         = new JPanel();

    JPanel pnl               = new JPanel();
    JPanel pnl2              = new JPanel();

    JPanel pnlFreq           = new JPanel();
    JPanel pnlWavelets       = new JPanel();

//    JPanel pnlB              = new JPanel();
//    JPanel pnlBarometric     = new JPanel();

    // Build Panel

    this.setLayout(new BorderLayout());

    // Panel Center

    pnlCenter.setLayout(new BorderLayout());

    // -- Track 1 (Seismic Wave) Panel

    pnlTrack1 = BuildTrack1LimitsPanel();

    // Barometric Pressure Panel

//    pnlB.setLayout(new BorderLayout());

//    if (stBaro != null)
//    {
//      pnlBarometric = BuildBarometricLimitsPanel();
//	}

    // Convolve Panel

    pnl.setLayout(new BorderLayout());

//    pnlWave = new psiSmoothingPanel( stCMN, notifier );
//    pnlWave.setData( stData );

    // Frequency Button Panel

    pnlFreq.setLayout(new GridLayout());

    btnFilter.setFont(new java.awt.Font("Dialog", 1, 11));
    btnFilter.setText("Filter Raw Data");
    btnFilter.addActionListener(this);

    btnFreq.setFont(new java.awt.Font("Dialog", 1, 11));
    btnFreq.setText("Set Tidal Pressure");
    btnFreq.addActionListener(this);

    pnlWavelets.setLayout(new GridLayout());

    btnWave.setFont(new java.awt.Font("Dialog", 1, 11));
    btnWave.setText("Create Wavelet Plot");
    btnWave.addActionListener(this);

    // Frequency Panel

    pnl2.setLayout(new BorderLayout());

    // Add Widgets to Frame

    this.add(pnlCenter,      BorderLayout.CENTER);
    pnlCenter.add(pnlTrack1, BorderLayout.NORTH);

//    pnlCenter.add(pnl,       BorderLayout.CENTER);
//    pnl.add(pnlWave,         BorderLayout.NORTH);

//    pnl.add(pnl2,            BorderLayout.CENTER);
//    pnl2.add(pnlFreq,        BorderLayout.NORTH);
//    pnlFreq.add(btnFilter,   null);
//    pnlFreq.add(btnFreq,     null);

//    pnl2.add(pnlB,           BorderLayout.CENTER);
//    if (stBaro != null)
//      pnlB.add(pnlBarometric,BorderLayout.NORTH);

//    pnl2.add(pnlWavelets,    BorderLayout.SOUTH);
//    pnlWavelets.add(btnWave, null);
  }

  /** Method BuildTrack1LimitsPanel()
   *  <p> this method will build the Track 1 Data Limits Panel
   */

  private JPanel BuildTrack1LimitsPanel()
  {
    JPanel pnlTrack         = new JPanel();
    JPanel pnlMinimum       = new JPanel();
    JPanel pnlMaximum       = new JPanel();

    TitledBorder titledBorderWaves = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        seismic.seisTracksStruct.SEIS_TRACK_DESC[3]+" Plot Track");
    titledBorderWaves.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderMinimum = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum");
    titledBorderMinimum.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMaximum = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum");
    titledBorderMaximum.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // -- Track 1 (Seismic Wave) Panel

    pnlTrack.setLayout(new GridLayout());
    pnlTrack.setBorder(titledBorderWaves);

    pnlMinimum.setLayout(new BorderLayout());
    pnlMinimum.setBorder(titledBorderMinimum);
    pnlMinimum.add(txtMinimum1);

    txtMinimum1.setText("");
	txtMinimum1.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMinimum1.addFocusListener(new psiControlPanelFocusAdapter(this));

    pnlMaximum.setLayout(new BorderLayout());
    pnlMaximum.setBorder(titledBorderMaximum);
    pnlMaximum.add(txtMaximum1);

    txtMaximum1.setText("");
	txtMaximum1.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMaximum1.addFocusListener(new psiControlPanelFocusAdapter(this));

    pnlTrack.add(pnlMinimum, null);
    pnlTrack.add(pnlMaximum, null);

    return (pnlTrack);
  }

  /** Method BuildBarometricLimitsPanel()
   *  <p> this method will build the Barometric Limits Panel
   *

  private JPanel BuildBarometricLimitsPanel()
  {
    JPanel pnlTrack         = new JPanel();
    JPanel pnlMinimum       = new JPanel();
    JPanel pnlMaximum       = new JPanel();

    TitledBorder titledBorderWaves = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Barometric Pressure Plot Track");
    titledBorderWaves.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderMinimum = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum");
    titledBorderMinimum.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMaximum = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum");
    titledBorderMaximum.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // -- Barometric Panel

    pnlTrack.setLayout(new GridLayout());
    pnlTrack.setBorder(titledBorderWaves);

    pnlMinimum.setLayout(new BorderLayout());
    pnlMinimum.setBorder(titledBorderMinimum);
    pnlMinimum.add(txtMinimumB);

    txtMinimumB.setText("");
	txtMinimumB.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMinimumB.addFocusListener(new psiControlPanelFocusAdapter(this));

    pnlMaximum.setLayout(new BorderLayout());
    pnlMaximum.setBorder(titledBorderMaximum);
    pnlMaximum.add(txtMaximumB);

    txtMaximumB.setText("");
	txtMaximumB.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMaximumB.addFocusListener(new psiControlPanelFocusAdapter(this));

    pnlTrack.add(pnlMinimum, null);
    pnlTrack.add(pnlMaximum, null);

    return (pnlTrack);
  }
*/
  /* ===================================================================== *
   * ----------------------------- CLOSE --------------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    stCMN     = null; // Global IQSTRAT Data Structure
    notifier  = null; // Observables Object
    stData    = null; // Seismic Data Structure
    stMOD     = null; // Modified to Mean Value
    stBaro    = null; // Barometric Pressure - Seismic Data Structure

    // Global Variables

    // Time Scale

    tsStart   = null; // Starting Date
    tsEnd     = null; // Ending Date

//    if (pnlWave != null)
//      pnlWave.close();
//    pnlWave   = null; // Construct Wave Panel

//    if (pnlFrequency != null)
//      pnlFrequency.close();
//    pnlFrequency = null; // Convolve Panel

    plotframe = null; // Seismic Data Plot Frame

    // Global Frame Widgets

    // -- Track 1 Minimum & Maximum Limits Widgets

    txtMinimum1 = null; // Minimum Value
    txtMaximum1 = null; // Maximum Value

    // -- Barometric Minimum & Maximum Limits Widgets

    txtMinimumB = null; // Minimum Value
    txtMaximumB = null; // Maximum Value

    // -- Frequency Plot Button

    btnFilter   = null; // Filter Data Button
    btnFreq     = null; // Frequecy Plot Button
    btnWave     = null; // Wavelet Plot Button
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getAvgData()
   *  <p> This method will retrieve the Averaged Data Set
   *  @return stAvg = the averaged seismic data structure
   *

  public seisStruct getAvgData() { return (pnlWave.getData()); }
*/
  /** Method compute()
   *  <p> This method can perform a default smoothing of the raw data
   */

  public void compute()
  {
	seisStruct stAvg = null;

//	pnlWave.Compute();

//	stAvg = getAvgData();
////    stMOD = seismic.seisUtility.resample(stAvg, stAvg.tsStart, stAvg.tsEnd, 60);

////    plotframe.setData(_PRESSURE_MOD, stMOD); // Modified Seismic Data structure
////    plotframe.refresh();
  }

  /** Method getModifiedData()
   *  <p> This method will retrieve the Averaged Data Set
   *  @return stAvg = the averaged seismic data structure
   *

  public void getModifiedData()
  {
	if (pnlFrequency != null)
	{
	  stMOD = pnlFrequency.getData();

      plotframe.setData(_PRESSURE_MOD, stMOD); // Modified Seismic Data structure
	  plotframe.setTidal(pnlFrequency.getTidal());
      plotframe.refresh();
	}
  }
*/
  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setPlot()
   *  <p> This method will pass the plotframe pointer
   *  @plotframe = the plot frame pointer
   */

  public void setPlot(seisPlotFrame plotframe)
  {
	this.plotframe = plotframe;
//    pnlWave.setPlot( plotframe );
  }

  /** Method setDateRange()
   *  <p> This method will set the depth range
   *

  public void setDateRange(Timestamp tsStart, Timestamp tsEnd)
  {
    this.tsStart = new Timestamp(tsStart.getTime()); // Starting Date
    this.tsEnd   = new Timestamp(tsEnd.getTime());   // Ending Date

    if (pnlFrequency != null)
    {
      pnlFrequency.setDateRange( tsStart, tsEnd );
	}
  }
*/
  /** Method shareData()
   *  <p> This method will share the Average Data
   *

  public void shareData()
  {
    if ((pnlFrequency != null) && (pnlWave != null))
	{
//	  pnlFrequency.setTitle(pnlWave.getTitle());
//	  pnlFrequency.setAvgData(pnlWave.getData());
	}
  }
*/
  /* ===================================================================== *
   * ------------------ FREQUENCY CALCULATOR ACTION ---------------------- *
   * ===================================================================== */

  /** Method displayFrequency()
   *  <p> This method will display the frequency calculator frame
   *

  private void displayFrequency()
  {
	if (pnlFrequency != null)
	  pnlFrequency.close();
	pnlFrequency = null;

    pnlFrequency = new psiTidesFrame(notifier, stCMN,
                                     seismic.seisUtility.copyData(getAvgData(), tsStart, tsEnd, 0));
//                     seismic.seisUtility.resample(getAvgData(), tsStart, tsEnd, 60));

    pnlFrequency.setPlot(plotframe);
    pnlFrequency.setDateRange( tsStart, tsEnd);
  }
*/
  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
//	if (event.getSource() == btnFilter) { pnlWave.displayFilterFrame(); }
//    if (event.getSource() == btnFreq)   { displayFrequency(); }
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
	focusLostTrack1(e);
	focusLostBarometricTrack(e);
  }

  /** METHOD focusLostTrack1()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostTrack1(FocusEvent e)
  {
    int     iNumeric = 0;
    int     len      = 0;
    int     iField   = -1;
    boolean bVal     = true;
    String  sMessage = new String("");
    String  sTemp    = new String("");
    int     id       = 0;
    double  d        = 0.0;
    double  d1       = 0.0;

    if (e.getSource() == txtMinimum1)
    {
      iData    = MINIMUM;
      iNumeric = 1;
      sTemp    = new String( txtMinimum1.getText().trim());
      sMessage = new String("Minimum Value is a Numeric Field");
    }

    if (e.getSource() == txtMaximum1)
    {
      iData    = MAXIMUM;
      iNumeric = 1;
      sTemp    = new String( txtMaximum1.getText().trim());
      sMessage = new String("Maximum Value is a Numeric Field");
    }

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);

        switch (iData)
       {
          case MINIMUM:
            txtMinimum1.setText(""+stData.dMinimum);
            break;

          case MAXIMUM:
            txtMaximum1.setText(""+stData.dMaximum);
            break;
	    }
      }
      else
      {
        switch (iData)
        {
          case MINIMUM:
          case MAXIMUM:
            d  = cmn.cmnString.stringToDouble(txtMinimum1.getText());
            d1 = cmn.cmnString.stringToDouble(txtMaximum1.getText());
            plotframe.setLimits( _PRESSURE_TEST, d, d1 );
            break;
	    }
      }
    }
  }

  /** METHOD focusLostBarometricTrack()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostBarometricTrack(FocusEvent e)
  {
    int     iNumeric = 0;
    int     len      = 0;
    int     iField   = -1;
    boolean bVal     = true;
    String  sMessage = new String("");
    String  sTemp    = new String("");
    int     id       = 0;
    double  d        = 0.0;
    double  d1       = 0.0;

    if (e.getSource() == txtMinimumB)
    {
      iData    = MINIMUM;
      iNumeric = 1;
      sTemp    = new String( txtMinimumB.getText().trim());
      sMessage = new String("Minimum Value is a Numeric Field");
    }

    if (e.getSource() == txtMaximumB)
    {
      iData    = MAXIMUM;
      iNumeric = 1;
      sTemp    = new String( txtMaximumB.getText().trim());
      sMessage = new String("Maximum Value is a Numeric Field");
    }

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);

        switch (iData)
       {
          case MINIMUM:
            txtMinimumB.setText(""+stBaro.dMinimum);
            break;

          case MAXIMUM:
            txtMaximumB.setText(""+stBaro.dMaximum);
            break;
	    }
      }
      else
      {
        switch (iData)
        {
          case MINIMUM:
          case MAXIMUM:
            d  = cmn.cmnString.stringToDouble(txtMinimumB.getText());
            d1 = cmn.cmnString.stringToDouble(txtMaximumB.getText());
            plotframe.setLimits( _PRESSURE_BARO, d, d1 );
            break;
	    }
      }
    }
  }
}

/** CLASS psiControlPanelFocusAdapter()
 *  <p> This Class will handle Actions when focus is lost on a textfield
 */

class psiControlPanelFocusAdapter extends java.awt.event.FocusAdapter
{
  psiControlPanel adaptee;

  psiControlPanelFocusAdapter(psiControlPanel adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/*
 *  @version 1.1 05/11/2016
 *  @author  John Victorine
 */
