/*
 * @psiPlotStruct.java Version 1.1 06/02/2017
 *
 * Copyright (c) 2017 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package psi;

import java.sql.Timestamp;

/** Class psiPlotStruct
 *  <p> This Class holds the information that was parsed from the miniSeed
 *      file using the IRIS (Incorporated Research Institutions
 *      for Seismology) source code.
 *
 *  @version 1.1 06/02/2017
 *  @author  John R. Victorine
 */

public class psiPlotStruct
{
  // Identifying information for the pressure Data

  public String    sName     = "";    // Name of pressure Data
  public String    sAPI      = "";    // API-Number

  // Plot Limits

  public Timestamp tsStart   = null; // Starting Timestamp
  public Timestamp tsEnd     = null; // Ending Timestamp

  public double    dMaximum  = 0.0;  // Maximum Data Value
  public double    dMinimum  = 0.0;  // Minimum Data Value

  // Plot Dimensions

  public int       iWidth    = 1200; // Actual Plotting Width
  public int       iHeight   =  800; // Actual plotting Height

  // Seismic Records

  public int       iRows     = 0;    // Total Number of rows
  public Timestamp ts[]      = null; // Time Stamp of data records
  public double    dRaw[]    = null; // Raw Data records

  /** Method delete()
   *  <p> This method deallocates the data to force Java to release memory
   *      allocated for the data
   */

  public void delete()
  {
    sName    = null;    // Name of pressure Data
    sAPI     = null;    // API-Number

    // Plot Limits

    tsStart  = null; // Starting Timestamp
    tsEnd    = null; // Ending Timestamp

    // Seismic Records

    iRows    = 0;    // Total Number of rows
    ts       = null; // Time Stamp of data record
    dRaw     = null; // Raw Data records
  }
}

/*
 *  @version 1.1 06/02/2017
 *  @author  John Victorine
 */
