/*
 * @psiPlotUtility.java Version 1.1 06/02/2017
 *
 * Copyright (c) 2017 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package psi;

import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import iqstrat.iqstratHeadersStruct; // Well Header Data Structure
import psi.psiPlotStruct;            // pressure Plot Data Structure
import seismic.seisStruct;           // Seismic Data Structure

/** Class psiPlotUtility
 *  <p> This is the psi data structure utility class.
 *
 *  @version 1.1 06/02/2017
 *  @author  John R. Victorine
 */

public class psiPlotUtility
{
  /** Method psiPlotStruct copy(psiPlotStruct stOld)
   *  <p> This method will copy the seismic data structure
   *  @param  stOld = Old psi data structure
   *  @return stNew = New psi data structure
   */

  public static psiPlotStruct copy(psiPlotStruct stOld)
  {
	psiPlotStruct stNew = null;

	if (stOld != null)
	{
	  stNew            = new psiPlotStruct();

      stNew.sName      = new String(stOld.sName);     // Name of pressure Data
      stNew.sAPI       = new String(stOld.sAPI);      // API-Number

      stNew.tsStart    = new Timestamp(stOld.tsStart.getTime());
      stNew.tsEnd      = new Timestamp(stOld.tsEnd.getTime());

      stNew.dMaximum   = stOld.dMaximum;             // Maximum Data Value
      stNew.dMinimum   = stOld.dMinimum;             // Minimum Data Value

      // Plot Dimensions

      stNew.iWidth     = stOld.iWidth;  // Actual Plotting Width
      stNew.iHeight    = stOld.iHeight; // Actual plotting Height

      // Records

      stNew.iRows      = stOld.iRows;                // Total Number of rows
      stNew.ts         = new Timestamp[stNew.iRows]; // Time Stamp of data records
      stNew.dRaw       = new double[stNew.iRows];    // Raw Data records

	  if (stNew.iRows > 0)
	  {
		for (int i=0; i<stNew.iRows; i++)
		{
		  stNew.ts[i]         = new Timestamp(stOld.ts[i].getTime());
		  stNew.dRaw[i]       = stOld.dRaw[i];
		}
	  }
	}

	return (stNew);
  }

  /** Method psiPlotStruct getPlotData(seisStruct stPSI)
   *  <p> This method will copy the seismic data structure
   *  @param  stPSI    = pressure data structure
   *  @return stPlot   = psi plot data structure
   */

  public static psiPlotStruct getPlotData(iqstratHeadersStruct stWell, seisStruct stPSI)
  {
	psiPlotStruct stPlot = null;
	int                  iCount = 0;
    int                  iStart = 0;
    int                  iNew   = 1;
    int                  iTotal = 0;

	if (stPSI != null)
	{
	  stPlot          = new psiPlotStruct();

      if (stWell != null)
      {
        stPlot.sName    = new String(stWell.sName);    // Name of pressure Data
        stPlot.sAPI     = new String(stWell.sAPI);     // API-Number
	  }

      stPlot.ts       = new Timestamp[stPSI.iRows]; // Time Stamp of data record
      stPlot.dRaw     = new double[stPSI.iRows];    // Raw Data record

      iTotal = 0;
      for (int i=0; i<stPSI.iRows; i++)
      {
		iTotal++;
  		if (iStart==0)
		{
          iStart++;
          iNew = 0;
		}

        if (iCount < stPSI.iRows)
  	    {
          stPlot.ts[iCount]    = new Timestamp(stPSI.ts[i].getTime());
          stPlot.dRaw[iCount]  = stPSI.data[i]; // pressure Data

          if (iNew==0)
          {
            stPlot.dMaximum = 2125.0; //stPlot.dRaw[iCount]; // Maximum Data Value
            stPlot.dMinimum = 2121.0; //stPlot.dRaw[iCount]; // Minimum Data Value
		    stPlot.tsStart  = new Timestamp(stPlot.ts[iCount].getTime()); // Starting Date
		    stPlot.tsEnd    = new Timestamp(stPlot.ts[iCount].getTime()); // Ending Date
		    iNew = 1;
		  }
		  else
		  {
		    stPlot.tsEnd    = new Timestamp(stPlot.ts[iCount].getTime()); // Ending Date
		  }

//		    if (stPlot.dMinimum > stPlot.dRaw[iCount])
//		      stPlot.dMinimum = stPlot.dRaw[iCount];

//		    if (stPlot.dMaximum < stPlot.dRaw[iCount])
//		      stPlot.dMaximum = stPlot.dRaw[iCount];

		  iCount++;
	    }
	  }

	  stPlot.iRows = iCount;
	}

	return (stPlot);
  }

  /** Method print()
   *  <p> This method will print the psi Data Structure
   */

  public static void print(psiPlotStruct st)
  {
    if (st != null)
    {
      for (int i=0; i<st.iRows; i++)
      {
        System.out.println(st.ts[i].toString()+" "+st.dRaw[i]);
      }

      System.out.print("Min="+st.dMinimum+" Max="+st.dMaximum);
      System.out.println(" Start="+st.tsStart.toString()+" End="+st.tsEnd.toString());
    }
  }
}

/*
 *  @version 1.1 06/02/2017
 *  @author  John Victorine
 */
