/*
 * @psiTidalStruct.java Version 1.1 07/22/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package psi;

import java.sql.Timestamp;

import seismic.seisStruct;  // Seismic Data Structure

/** Class psiTidalStruct
 *  <p> This is the seismic data structure utility class for
 *      IRIS (Incorporated Research Institutions for Seismology) Data.
 *
 *  @version 1.1 07/22/2016
 *  @author  John R. Victorine
 */

public class psiTidalStruct
{
  public static final int _MEAN   = 0;
  public static final int _LINEAR = 1;
  public static final int _DIFF   = 2;

  public static final int DEG_F   = 0;
  public static final int DEG_C   = 1;

  public static final int DIURNAL      = 0;
  public static final int SEMI_DIURNAL = 1;
//  public static final int DIURNAL      = psi.gui.psiFrequencyTable.DIURNAL;
//  public static final int SEMI_DIURNAL = psi.gui.psiFrequencyTable.SEMI_DIURNAL;

  // -- Pressure Values

  public int iSubtract = _LINEAR;

  public double dMean  = 0.0;  // Mean pressure value of data set.
  public double dStart = 0.0;  // First 100 pressure data points averaged.
  public double dEnd   = 0.0;  // Last 100 pressure data points averaged.

  // -- Aquifer Values

  public int    iTemp  = DEG_F;

  public double dCw    = 0.0;   // GPa
  public double dPHI   = 0.1;   // PU
  public double dTemp  = 130.0; // deg F

  // -- Potential Values

  public double dWell  = 0.0;   // Latitude - Well
  public double dMoon  = 30.0;  // Latitude - Moon

  // .. Diurnal

  public double dFd    = 0.0;   // Frequency [Hz]
  public double dPHId  = 0.0;   // Phase [degrees]

  // .. Semi-diurnal

  public double dFs    = 0.0;   // Frequency [Hz]
  public double dPHIs  = 0.0;   // Phase [degrees]
}

/*
 *  @version 1.1 07/22/2016
 *  @author  John Victorine
 */
