/*
 * @seisUtility.java Version 1.1 03/03/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import seismic.seisStruct;  // Seismic Data Structure

/** Class seisUtility
 *  <p> This is the seismic data structure utility class for
 *      IRIS (Incorporated Research Institutions for Seismology) Data.
 *
 *  @version 1.1 03/03/2015
 *  @author  John R. Victorine
 */

public class seisUtility
{
  /** Method seisStruct copyData()
   *  <p> This method will copy the seismic data structure
   *  @param  oldStruct = Old seismic data structure
   *  @return newStruct = New seismic data structure
   */

  public static seisStruct copyData(seisStruct stOld)
  {
	seisStruct stNew = null;

	if (stOld != null)
	{
	  stNew          = new seisStruct();

      stNew.network  = new String(stOld.network);  // Network Code
      stNew.station  = new String(stOld.station);  // Station Identifier
      stNew.location = new String(stOld.location); // Location Identifier
      stNew.channel  = new String(stOld.channel);  // Channel Identifier

      stNew.sData    = new String(stOld.sData);    // Type of Filter
      stNew.sData2   = new String(stOld.sData2);   // Specifics of Filtering
      stNew.sData3   = new String(stOld.sData3);   // Specifics of Filtering

      stNew.tsStart  = new Timestamp(stOld.tsStart.getTime());
      stNew.tsEnd    = new Timestamp(stOld.tsEnd.getTime());

      stNew.dMaximum = stOld.dMaximum; // Maximum Data Value
      stNew.dMinimum = stOld.dMinimum; // Minimum Data Value

      // Seismic Records

      stNew.iRows = stOld.iRows;                   // Total Number of rows
      stNew.ts    = new Timestamp[stNew.iRows];    // Time Stamp of data record
      stNew.data  = new double[stNew.iRows];       // Data record

	  if (stNew.iRows > 0)
	  {
		for (int i=0; i<stNew.iRows; i++)
		{
		  stNew.ts[i]   = new Timestamp(stOld.ts[i].getTime());
		  stNew.data[i] = stOld.data[i];
//System.out.println(stNew.iRows+" "+i+" "+stNew.ts[i].toString()+" "+stNew.data[i]);
		}
	  }
	}

	return (stNew);
  }

  /** Method seisStruct resample()
   *  <p> This method will copy the seismic data structure
   *  @param  oldStruct = Old seismic data structure
   *  @param  iMinutes  = the number of minutes to resample with
   *  @return newStruct = New seismic data structure
   */

  public static seisStruct resample(seisStruct stOld, int iMinutes)
  {
	int i=0;
	seisStruct stNew    = null;
	int        iSeconds = 0;
	int        iCount   = 0;
	int        iTotal   = 0;

	if (stOld != null)
	{
	  iSeconds       = 60 * iMinutes;

  	  stNew          = new seisStruct();

      stNew.network  = new String(stOld.network);  // Network Code
      stNew.station  = new String(stOld.station);  // Station Identifier
      stNew.location = new String(stOld.location); // Location Identifier
      stNew.channel  = new String(stOld.channel);  // Channel Identifier

      stNew.sData    = new String(stOld.sData);    // Type of Filter
      stNew.sData2   = new String(stOld.sData2);   // Specifics of Filtering
      stNew.sData3   = new String(stOld.sData3);   // Specifics of Filtering

      stNew.tsStart  = new Timestamp(stOld.tsStart.getTime());
      stNew.tsEnd    = new Timestamp(stOld.tsEnd.getTime());

      stNew.dMaximum = stOld.dMaximum; // Maximum Data Value
      stNew.dMinimum = stOld.dMinimum; // Minimum Data Value

      // Pressure Records

      // Count the total records that will be added to the Pressure Structure

      i = 0;

	  if (stOld.iRows > 0)
	  {
		while (i<stOld.iRows)
		{
		  i = i+iSeconds;
		  iTotal++;
		}
	  }

      // Allocate Memory for the Pressure Data

      stNew.ts    = new Timestamp[iTotal];    // Time Stamp of data record
      stNew.data  = new double[iTotal];       // Data record

      // Retrieve the Pressure data into the data structure

      i = 0;

	  if (stOld.iRows > 0)
	  {
		while (i<stOld.iRows)
		{
		  if (iCount < iTotal)
		  {
		    stNew.ts[iCount]   = new Timestamp(stOld.ts[i].getTime());
		    stNew.data[iCount] = stOld.data[i];
		    iCount++;
		  }

		  i = i+iSeconds;
		}
        stNew.iRows = iCount;  // Total Number of rows
	  }

	  stOld.delete();
	  stOld = null;
	}

	return (stNew);
  }

  /** Method seisStruct resample()
   *  <p> This method will copy the seismic data structure
   *  @param  oldStruct = Old seismic data structure
   *  @param  iMinutes  = the number of minutes to resample with
   *  @param  corr      = pressure correction to the Raw data
   *  @return newStruct = New seismic data structure
   */

  public static seisStruct resample(seisStruct stOld, int iMinutes, double corr)
  {
	int i=0;
	seisStruct stNew    = null;
	int        iSeconds = 0;
	int        iCount   = 0;
	int        iTotal   = 0;

	if (stOld != null)
	{
	  iSeconds       = 60 * iMinutes;

  	  stNew          = new seisStruct();

      stNew.network  = new String(stOld.network);  // Network Code
      stNew.station  = new String(stOld.station);  // Station Identifier
      stNew.location = new String(stOld.location); // Location Identifier
      stNew.channel  = new String(stOld.channel);  // Channel Identifier

      stNew.sData    = new String(stOld.sData);    // Type of Filter
      stNew.sData2   = new String(stOld.sData2);   // Specifics of Filtering
      stNew.sData3   = new String(stOld.sData3);   // Specifics of Filtering

      stNew.tsStart  = new Timestamp(stOld.tsStart.getTime());
      stNew.tsEnd    = new Timestamp(stOld.tsEnd.getTime());

      stNew.dMaximum = stOld.dMaximum; // Maximum Data Value
      stNew.dMinimum = stOld.dMinimum; // Minimum Data Value

      // Pressure Records

      // Count the total records that will be added to the Pressure Structure

      i = 0;

	  if (stOld.iRows > 0)
	  {
		while (i<stOld.iRows)
		{
		  i = i+iSeconds;
		  iTotal++;
		}
	  }

      // Allocate Memory for the Pressure Data

      stNew.ts    = new Timestamp[iTotal];    // Time Stamp of data record
      stNew.data  = new double[iTotal];       // Data record

      // Retrieve the Pressure data into the data structure

      i = 0;

	  if (stOld.iRows > 0)
	  {
		while (i<stOld.iRows)
		{
		  if (iCount < iTotal)
		  {
		    stNew.ts[iCount]   = new Timestamp(stOld.ts[i].getTime());
		    stNew.data[iCount] = stOld.data[i] + corr;
		    iCount++;
		  }

		  i = i+iSeconds;
		}
        stNew.iRows = iCount;  // Total Number of rows
	  }

	  stOld.delete();
	  stOld = null;
	}

	return (stNew);
  }

  /** Method seisStruct transfer()
   *  <p> This method will copy the seismic data structure and delete the old structure
   *  @param  oldStruct = Old seismic data structure
   *  @return newStruct = New seismic data structure
   */

  public static seisStruct transfer(seisStruct stOld)
  {
	seisStruct stNew  = null;

	if (stOld != null)
	{
 	  stNew = copyData( stOld );

 	  stOld.delete();
 	  stOld = null;
	}

	return (stNew);
  }

  /** Method seisStruct copyData()
   *  <p> This method will copy the seismic data structure
   *  @param  oldStruct = Old seismic data structure
   *  @param  stStart   = Starting Date & Time of data set
   *  @param  stEnd     = Ending Date & Time of data set
   *  @return newStruct = New seismic data structure
   */

  public static seisStruct copyData(seisStruct stOld, Timestamp tsStart, Timestamp tsEnd, int iHours)
  {
	seisStruct stNew  = null;
	int        iCount = 0;
	long       lstart = (long) 0;
	long       lend   = (long) 0;
	long       ltime  = (long) 0;
	int        iStart = 0;
	long       lhours = (long) 3600000;
    Timestamp  tsTemp = null;

	if (stOld != null)
	{
	  stNew          = new seisStruct();

      stNew.network  = new String(stOld.network);  // Network Code
      stNew.station  = new String(stOld.station);  // Station Identifier
      stNew.location = new String(stOld.location); // Location Identifier
      stNew.channel  = new String(stOld.channel);  // Channel Identifier

      stNew.sData    = new String(stOld.sData);    // Type of Filter
      stNew.sData2   = new String(stOld.sData2);   // Specifics of Filtering
      stNew.sData3   = new String(stOld.sData3);   // Specifics of Filtering

      stNew.tsStart  = new Timestamp(tsStart.getTime());
      stNew.tsEnd    = new Timestamp(tsEnd.getTime());

      stNew.ts       = new Timestamp[stOld.iRows];    // Time Stamp of data record
      stNew.data     = new double[stOld.iRows];       // Data record

	  if (stOld.iRows > 0)
	  {
        lstart = tsStart.getTime();
        lend   = tsEnd.getTime();

		for (int i=0; i<stOld.iRows; i++)
		{
		  ltime = stOld.ts[i].getTime() + iHours * lhours;

		  if ((ltime >= lstart) && (ltime <= lend))
		  {
	        tsTemp = new Timestamp(ltime);
//if (iCount == 0)
//System.out.println(	stOld.ts[i].toString()+" "+tsTemp.toString());

//		    stNew.ts[iCount]   = new Timestamp(stOld.ts[i].getTime());
		    stNew.ts[iCount]   = new Timestamp(ltime);
		    stNew.data[iCount] = stOld.data[i];

  		    if (iStart == 0)
  		    {
		  	  stNew.dMinimum = stNew.data[iCount];
			  stNew.dMaximum = stNew.data[iCount];
			  iStart = 1;
		    }

		    if (stNew.dMinimum > stNew.data[iCount])
		      stNew.dMinimum = stNew.data[iCount];

		    if (stNew.dMaximum < stNew.data[iCount])
		      stNew.dMaximum = stNew.data[iCount];

		    iCount++;
		  }
		}
	  }
      stNew.iRows = iCount;
   	}

	return (stNew);
  }

  /** Method seisStruct resample()
   *  <p> This method will copy the seismic data structure
   *  @param  oldStruct = Old seismic data structure
   *  @param  stStart   = Starting Date & Time of data set
   *  @param  stEnd     = Ending Date & Time of data set
   *  @return newStruct = New seismic data structure
   */

  public static seisStruct resample(seisStruct stOld, Timestamp tsStart, Timestamp tsEnd, int iSeconds)
  {
	int        i      = 0;
	seisStruct stNew  = null;
	int        iCount = 0;
	long       lstart = (long) 0;
	long       lend   = (long) 0;
	long       ltime  = (long) 0;
	int        iStart = 0;
	double     dSum   = 0.0;
	double     dMean  = 0.0;

	if (stOld != null)
	{
	  stNew          = new seisStruct();

      stNew.network  = new String(stOld.network);  // Network Code
      stNew.station  = new String(stOld.station);  // Station Identifier
      stNew.location = new String(stOld.location); // Location Identifier
      stNew.channel  = new String(stOld.channel);  // Channel Identifier

      stNew.sData    = new String(stOld.sData);    // Type of Filter
      stNew.sData2   = new String(stOld.sData2);   // Specifics of Filtering
      stNew.sData3   = new String(stOld.sData3);   // Specifics of Filtering

      stNew.tsStart  = new Timestamp(tsStart.getTime());
      stNew.tsEnd    = new Timestamp(tsEnd.getTime());

      stNew.ts       = new Timestamp[stOld.iRows];    // Time Stamp of data record
      stNew.data     = new double[stOld.iRows];       // Data record

	  if (stOld.iRows > 0)
	  {
		for (i=0; i<stOld.iRows; i++)
		{
		  dSum = dSum + stOld.data[i];
		}

		dMean = dSum/stOld.iRows;
	  }

      i = 0;

	  if (stOld.iRows > 0)
	  {
        lstart = tsStart.getTime();
        lend   = tsEnd.getTime();

//		for (int i=0; i<stOld.iRows; i++)
		while (i<stOld.iRows)
		{
		  ltime = stOld.ts[i].getTime();

		  if ((ltime >= lstart) && (ltime <= lend))
		  {
		    stNew.tsEnd        = new Timestamp(ltime);

		    stNew.ts[iCount]   = new Timestamp(ltime);
		    stNew.data[iCount] = stOld.data[i] - dMean;

  		    if (iStart == 0)
  		    {
		  	  stNew.dMinimum = stNew.data[iCount];
			  stNew.dMaximum = stNew.data[iCount];
			  iStart = 1;
		    }

		    if (stNew.dMinimum > stNew.data[iCount])
		      stNew.dMinimum = stNew.data[iCount];

		    if (stNew.dMaximum < stNew.data[iCount])
		      stNew.dMaximum = stNew.data[iCount];

            stNew.dMinimum = (double) ((int) (stNew.dMinimum *10000.0) / 10000.0);
            stNew.dMaximum = (double) ((int) (stNew.dMaximum *10000.0) / 10000.0);

		    iCount++;
		  }
		  i = i+iSeconds;
		}
	  }
      stNew.iRows = iCount;
   	}

	return (stNew);
  }

  /** Method seisStruct merge()
   *  <p> This method will copy the seismic data structure
   *  @param  st1   = 1st seismic data structure
   *  @param  st2   = 2nd seismic data structure
   *  @return stNew = New seismic data structure
   */

  public static seisStruct merge(seisStruct st1, seisStruct st2)
  {
	int        i     = 0;
	int        icnt  = 0;
	seisStruct stNew = null;

	if ((st1 != null) && (st2 != null))
	{
	  stNew          = new seisStruct();

      stNew.network  = new String(st1.network);  // Network Code
      stNew.station  = new String(st1.station);  // Station Identifier
      stNew.location = new String(st1.location); // Location Identifier
      stNew.channel  = new String(st1.channel);  // Channel Identifier

      stNew.sData    = new String(st1.sData);    // Type of Filter
      stNew.sData2   = new String(st1.sData2);   // Specifics of Filtering
      stNew.sData3   = new String(st1.sData3);   // Specifics of Filtering

      stNew.tsStart  = new Timestamp(st1.tsStart.getTime());
      stNew.tsEnd    = new Timestamp(st2.tsEnd.getTime());

      stNew.dMaximum = st1.dMaximum;             // Maximum Data Value
      stNew.dMinimum = st1.dMinimum;             // Minimum Data Value

      // Seismic Records

      stNew.iRows = st1.iRows + st2.iRows;       // Total Number of rows
      stNew.ts    = new Timestamp[stNew.iRows];  // Time Stamp of data record
      stNew.data  = new double[stNew.iRows];     // Data record

	  if (stNew.iRows > 0)
	  {
		icnt = 0;
		for (i=0; i<st1.iRows; i++)
		{
	      if (icnt < stNew.iRows)
	      {
		    stNew.ts[icnt]   = new Timestamp(st1.ts[i].getTime());
		    stNew.data[icnt] = st1.data[i];
		    icnt++;
		  }
		}

  	    st1.delete();
	    st1 = null;

		for (i=0; i<st2.iRows; i++)
		{
	      if (icnt < stNew.iRows)
	      {
  		    stNew.ts[icnt]   = new Timestamp(st2.ts[i].getTime());
		    stNew.data[icnt] = st2.data[i];

		    if (stNew.dMinimum > stNew.data[icnt])
		      stNew.dMinimum = stNew.data[icnt];

		    if (stNew.dMaximum < stNew.data[icnt])
		      stNew.dMaximum = stNew.data[icnt];

            stNew.dMinimum = (double) ((int) (stNew.dMinimum *10000.0) / 10000.0);
            stNew.dMaximum = (double) ((int) (stNew.dMaximum *10000.0) / 10000.0);

		    icnt++;
		  }
		}

  	    st2.delete();
	    st2 = null;

		stNew.iRows = icnt;
	  }
	}

	return (stNew);
  }
}

/*
 *  @version 1.1 03/03/2015
 *  @author  John Victorine
 */
